<?php
/**
 * UploadBehavior class file.
 * @author    Ahamed Rifaideen <rifajas@gmail.com>
 * @copyright Copyright &copy; Ahamed Rifaideen 2014
 * @license   http://www.opensource.org/licenses/bsd-license.php New BSD License
 */

/**
 * Upload Behavior manages file upload and other file related operations such as delete file when record is deleted,
 * creates unique file name when uploading files, delete previous file when new file is uploaded for existing file
 * and this behavior can be customized through its public properties.
 *
 * public function behaviors() {
 *
 * return array(
 * 'upload' => array(
 * 'class' => 'UploadBehavior',
 *  //default values
 * 'field' => 'file',
 * 'path' => 'assets',
 * 'renameIfExists' => true,
 * 'delete_previous_file' => true
 * 'delete_together' => true
 * )
 * );
 *
 * }
 *
 */
class UploadBehavior extends CActiveRecordBehavior
{

    /**
     * name of the file field that this behavior manages
     */
    public $field = 'file';
    /**
     * array of allowed extensions
     */
    public $allowed = array('pdf', 'PDF', 'JPG', 'JPEG', 'GIF', 'PNG', 'ZIP', 'jpg', 'jpeg', 'zip', 'png', 'gif');
    /**
     * name of the path where uploaded files are saved
     * Note: path must not end with slash ( / )
     */
    public $path = 'e-Teczka'.DIRECTORY_SEPARATOR.'B_w_trakcie_zatrudnienia'.DIRECTORY_SEPARATOR.'Chorobowe';
    /**
     * rename the file when the file already exists
     */
    public $renameIfExists = TRUE;
    /**
     * delete previous file when new file is uploaded
     */
    public $delete_previous_file = TRUE;
    /**
     * delete file together with record
     */
    public $delete_together = TRUE;

    /**
     * uploaded file extension
     */
    private $_extension;
    /**
     * error flag, default implementation will show error when this flag is true
     */
    private $hasError;
    /**
     * previous file name
     */
    private $previous_file;

    /**
     * assign file field to previous file
     *
     * @param CModelEvent $event
     */
    public function afterFind($event)
    {
        $this->previous_file = $this->owner->{$this->field};
    }

    public function calcPath()
    {
        $dir = new DirectoryHelper();
        $mainDir = $dir->mainDirectory();

        // uzytkownik zapisany w danym wniosku
        $tobj = Userfirma::model()->findByAttributes(array('userid' => $this->owner->{'userid'}));

        if (is_object($tobj)) {
            $kadryid = $tobj->kadryid;
            $userDir = KadryPracownicy::model()->userDir($kadryid, $this->owner->{'userid'});
        } else {
            $userDir = KadryPracownicy::model()->userDir(NULL, $this->owner->{'userid'});
        }

        return $mainDir . DIRECTORY_SEPARATOR . $userDir . DIRECTORY_SEPARATOR . $this->path;
    }

    /**
     * deletes the file after the record is deleted
     * it deletes only when delete_together is set to true
     *
     * @param CModelEvent $event
     */
    public function afterDelete($event)
    {

        if ($this->delete_together === TRUE) {
            $f = $this->calcPath() . DIRECTORY_SEPARATOR . $this->owner->{$this->field};

            if (file_exists($f)) {
                @unlink($f);
            }
        }
    }

    /**
     * check for file upload and save the file when uploaded
     */
    public function beforeSave($event)
    {

        if ($this->owner->{$this->field} instanceof CUploadedFile) {

            $file = $this->owner->{$this->field}->name;
            $file = iconv("UTF-8", "ASCII//TRANSLIT", $file);
            $this->getUniqueFileName($file);

            if ($this->hasError === TRUE) {
                $this->owner->addError($this->field, $file . ' już istnieje');
                $event->isValid = FALSE;
                return FALSE;
            }

            // Nowe - walidacja pliku
            $ext = CFileHelper::getExtension($file);
            if (!in_array($ext, $this->allowed)) {
                $extlist = implode(',', $this->allowed);
                $this->owner->addError($this->field, 'Niedozwolone rozszerzenie pliku. Dozwolone rozszerzenia: ' . $extlist);
                $event->isValid = FALSE;
                return FALSE;
            }

            if (!file_exists($this->calcPath())) {
                mkdir($this->calcPath(), 0777, TRUE);
            }

            $this->owner->{$this->field}->saveAs($this->calcPath() . DIRECTORY_SEPARATOR . $file);
            $this->owner->{$this->field} = $file;

            if ($this->delete_previous_file === TRUE) {
                $f = $this->calcPath() . DIRECTORY_SEPARATOR . $this->previous_file;

                if (file_exists($f) && $f != $file) {
                    @unlink($f);
                }
            }
        } else {
            // Wymagany plik jeśli Rodzaj wniosku ma zaznaczony plik do pobrania
            $obj = Rodzaj::model()->FindByPK($this->owner->{'rodzajid'});
            if (is_object($obj)) {
                if ($obj->pliki == 1 OR $obj->pliki == '1') {
                    if ($this->owner->{$this->field} === NULL) {
                        $this->owner->addError($this->field, 'Wymagane załączenie pliku przy wniosku chorobowym.');
                        $event->isValid = FALSE;
                        return FALSE;
                    }
                }
            }
        }
    }

    /**
     * load file instance before validates
     */
    public function beforeValidate($event)
    {
        $this->loadFile();
        if ($this->owner->{$this->field} === NULL) {
            $this->owner->{$this->field} = $this->previous_file;
        }
    }

    /**
     * load file instance
     */
    public function loadFile()
    {
        $this->owner->{$this->field} = CUploadedFile::getInstance($this->owner, $this->field);
    }

    /**
     * get unique file name.
     *
     * @param string $file
     *
     * @return boolean
     */
    public function getUniqueFileName(&$file)
    {
        if (file_exists($this->calcPath() . DIRECTORY_SEPARATOR . $file)) {
            if ($this->renameIfExists === TRUE) {
                $this->_extension = '.' . CFileHelper::getExtension($file);
                $file = md5(microtime() . date('mdyHi')) . $this->_extension;
                $this->getUniqueFileName($file);
            } else {
                $this->hasError = TRUE;
                return FALSE;
            }
        } else {
            $this->hasError = FALSE;
            return TRUE;
        }
    }

}
